==> itwo_flutter_base-0.0.3.tar.gz: lib/widget/widget_image_up.dart <==
import 'dart:ui' as ui;

import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

import 'widget_ext.dart';

class ImageGridView extends StatefulWidget {
  final IndexedWidgetBuilder itemBuilder;
  final int itemCount;
  final int? maxItemCount;
  final Widget? clearIcon;
  final WidgetBuilder? addItemBuilder;
  final int? crossAxisCount;
  final double? mainAxisSpacing;
  final double? crossAxisSpacing;
  final VoidCallback? onTapAdd;
  final Function(int)? onTapDel;

  ImageGridView(
    this.itemBuilder,
    this.itemCount, {
    this.clearIcon,
    this.maxItemCount,
    this.addItemBuilder,
    this.crossAxisCount,
    this.mainAxisSpacing,
    this.crossAxisSpacing,
    this.onTapAdd,
    this.onTapDel,
  });

  @override
  _ImageGridViewState createState() => _ImageGridViewState();
}

class _ImageGridViewState extends State<ImageGridView> {
  late double deviceWidth;
  late double itemSize;

  @override
  void initState() {
    super.initState();
    deviceWidth = MediaQueryData.fromWindow(ui.window).size.width;
  }

  @override
  Widget build(BuildContext context) {
    // Widget _clean = widget.clearIcon ?? _defaultCleanIcon;
    int crossAxisCount = widget.crossAxisCount ?? _defaultCrossAxisCount;
    // Widget addItem = widget.addItemBuilder?.call(context) ?? _defaultAddItemWidget;
    double mainAxisSpacing = widget.mainAxisSpacing ?? _defaultSpacing;
    double crossAxisSpacing = widget.crossAxisSpacing ?? _defaultSpacing;
    itemSize = deviceWidth - (crossAxisCount - 1) * crossAxisSpacing;
    return GridView.builder(
      gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: crossAxisCount, mainAxisSpacing: mainAxisSpacing, crossAxisSpacing: crossAxisSpacing),
      itemBuilder: _itemBuilder,
      itemCount: _itemCount(),
    );
  }

  Widget _itemBuilder(BuildContext context, int index) {
    if (index >= widget.itemCount) {
      var addItem = widget.addItemBuilder?.call(context) ?? _defaultAddItemWidget;
      return addItem.onTap(() => widget.onTapAdd?.call());
    }
    var item = widget.itemBuilder.call(context, index);
    return Stack(
      children: [
        SizedBox(width: itemSize, height: itemSize, child: item),
        Positioned(child: widget.clearIcon ?? _defaultCleanIcon, right: 0, top: 0).onTap(() => widget.onTapDel?.call(index)),
      ],
    );
  }

  int _itemCount() {
    if (widget.itemCount == widget.maxItemCount) {
      return widget.itemCount;
    }
    return widget.itemCount + 1;
  }

  // bool _isAddIconEnable() => widget.itemCount == widget.maxItemCount;

  Widget _defaultCleanIcon = Padding(padding: EdgeInsets.all(4), child: Icon(Icons.close, size: 16, color: Colors.red));
  Widget _defaultAddItemWidget = Container(
      child: Icon(Icons.add, size: 24, color: Colors.white),
      decoration: BoxDecoration(
        color: Colors.grey.shade200,
        borderRadius: BorderRadius.circular(8),
        border: Border.all(color: Colors.grey.shade300),
      ));
  int _defaultCrossAxisCount = 3;
  double _defaultSpacing = 10;
}

==> itwo_flutter_base-0.0.8.tar.gz: lib/widget/widget_image_up.dart <==
import 'dart:ui' as ui;

import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

class ImageGridView extends StatefulWidget {
  final IndexedWidgetBuilder itemBuilder;
  final int itemCount;
  final bool? sliver; // 是否是 sliver 组件
  final int? maxItemCount; // 最大的图片数量
  final Widget? clearIcon; // 删除按钮
  final WidgetBuilder? addItemBuilder; // 添加图片按钮
  final int? crossAxisCount; // 多少列
  final double? mainAxisSpacing; // 主轴间隔
  final double? crossAxisSpacing; // 纵轴间隔
  final VoidCallback? onTapAdd;
  final Function(int)? onTapDel;

  ImageGridView(
    this.itemBuilder,
    this.itemCount, {
    this.sliver,
    this.clearIcon,
    this.maxItemCount,
    this.addItemBuilder,
    this.crossAxisCount,
    this.mainAxisSpacing,
    this.crossAxisSpacing,
    this.onTapAdd,
    this.onTapDel,
  });

  @override
  _ImageGridViewState createState() => _ImageGridViewState();
}

class _ImageGridViewState extends State<ImageGridView> {
  late double deviceWidth;
  late double itemSize;

  @override
  void initState() {
    super.initState();
    deviceWidth = MediaQueryData.fromWindow(ui.window).size.width;
  }

  @override
  Widget build(BuildContext context) {
    // Widget _clean = widget.clearIcon ?? _defaultCleanIcon;
    int crossAxisCount = widget.crossAxisCount ?? _defaultCrossAxisCount;
    // Widget addItem = widget.addItemBuilder?.call(context) ?? _defaultAddItemWidget;
    double mainAxisSpacing = widget.mainAxisSpacing ?? _defaultSpacing;
    double crossAxisSpacing = widget.crossAxisSpacing ?? _defaultSpacing;
    itemSize = deviceWidth - (crossAxisCount - 1) * crossAxisSpacing;
    var sliverGridDelegateWithFixedCrossAxisCount = SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: crossAxisCount, mainAxisSpacing: mainAxisSpacing, crossAxisSpacing: crossAxisSpacing);
    return widget.sliver == true
        ? SliverGrid(
            delegate: SliverChildBuilderDelegate(
              _itemBuilder,
              childCount: _itemCount(),
            ),
            gridDelegate: sliverGridDelegateWithFixedCrossAxisCount,
          )
        : GridView.builder(
            physics: NeverScrollableScrollPhysics(),
            gridDelegate: sliverGridDelegateWithFixedCrossAxisCount,
            itemBuilder: _itemBuilder,
            shrinkWrap: true,
            itemCount: _itemCount(),
          );
  }

  Widget _itemBuilder(BuildContext context, int index) {
    if (index >= widget.itemCount) {
      var addItem = widget.addItemBuilder?.call(context) ?? _defaultAddItemWidget;
      return InkWell(onTap: () => widget.onTapAdd?.call(), child: addItem);
    }
    var item = widget.itemBuilder.call(context, index);
    var clear = InkWell(onTap: () => widget.onTapDel?.call(index), child: widget.clearIcon ?? _defaultCleanIcon);
    return Stack(
      children: [
        SizedBox(width: itemSize, height: itemSize, child: item),
        Container(child: clear, alignment: Alignment.topRight),
      ],
    );
  }

  int _itemCount() {
    if (widget.itemCount == widget.maxItemCount) {
      return widget.itemCount;
    }
    return widget.itemCount + 1;
  }

  // bool _isAddIconEnable() => widget.itemCount == widget.maxItemCount;

  Widget _defaultCleanIcon = Padding(padding: EdgeInsets.all(4), child: Icon(Icons.close, size: 16, color: Colors.red));
  Widget _defaultAddItemWidget = Container(
      child: Icon(Icons.add, size: 24, color: Colors.white),
      decoration: BoxDecoration(
        color: Colors.grey.shade200,
        borderRadius: BorderRadius.circular(8),
        border: Border.all(color: Colors.grey.shade300),
      ));
  int _defaultCrossAxisCount = 3;
  double _defaultSpacing = 10;
}
